#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

# file      : wind_main.py
# project   : xbpf
# copyright : (C) 2014 by Claude BULIN
#
# This file is part of bpf project
#
# xbpf is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; version 3.
#
# xbpf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA


import sys, os
try:
	from gi.repository import Gtk, GObject, Gdk
except:
	print("GTK Not Availible")
	sys.exit(1)
import subprocess, threading
from fconfig import FConfig
import Language_Init
from prefs import Preferences
from build_pot_file import build_pot_file
from wind_info import wind_info
from wind_about import wind_about


# constant variables for notebook
(NOTEBOOK_CHOICE, NOTEBOOK_CHOICE_TRANSLATE, NOTEBOOK_GENERATE) = range(3)

language_liste = [
	[ "checkbutton_afrikaans",  "afrikaans",  "af" ],
	[ "checkbutton_albanian",   "albanian",   "sq" ],
	[ "checkbutton_arabic",     "arabic",     "ar" ],
	[ "checkbutton_belarusian", "belarusian", "be" ],
	[ "checkbutton_bulgarian",  "bulgarian",  "bg" ],
	[ "checkbutton_catalan",    "catalan",    "ca" ],
	[ "checkbutton_chinese",    "chinese",    "zh" ],
	[ "checkbutton_croatian",   "croatian",   "hr" ],
	[ "checkbutton_czech",      "czech",      "cs" ],
	[ "checkbutton_danish",     "danish",     "da" ],

	[ "checkbutton_dutch",      "dutch",      "nl" ],
	[ "checkbutton_english",    "english",    "en" ],
	[ "checkbutton_estonian",   "estonian",   "et" ],
	[ "checkbutton_filipino",   "filipino",   "tl" ],
	[ "checkbutton_finnish",    "finnish",    "fi" ],
	[ "checkbutton_french",     "french",     "fr" ],
	[ "checkbutton_galician",   "galician",   "gl" ],
	[ "checkbutton_german",     "german",     "de" ],
	[ "checkbutton_greek",      "greek",      "el" ],
	[ "checkbutton_hebrew",     "hebrew",     "iw" ],

	[ "checkbutton_hindi",      "hindi",      "hi" ],
	[ "checkbutton_hungarian",  "hungarian",  "hu" ],
	[ "checkbutton_icelandic",  "icelandic",  "is" ],
	[ "checkbutton_indonesian", "indonesian", "id" ],
	[ "checkbutton_irish",      "irish",      "ga" ],
	[ "checkbutton_italian",    "italian",    "it" ],
	[ "checkbutton_japanese",   "japanese",   "ja" ],
	[ "checkbutton_korean",     "korean",     "ko" ],
	[ "checkbutton_latvian",    "latvian",    "lv" ],
	[ "checkbutton_lithuanian", "lithuanian", "lt" ],

	[ "checkbutton_macedonian", "macedonian", "mk" ],
	[ "checkbutton_malay",      "malay",      "ms" ],
	[ "checkbutton_maltese",    "maltese",    "mt" ],
	[ "checkbutton_norwegian",  "norwegian",  "no" ],
	[ "checkbutton_persian",    "persian",    "fa" ],
	[ "checkbutton_polish",     "polish",     "pl" ],
	[ "checkbutton_portuguese", "portuguese", "pt" ],
	[ "checkbutton_romanian",   "romanian",   "ro" ],
	[ "checkbutton_russian",    "russian",    "ru" ],
	[ "checkbutton_serbian",    "serbian",    "sr" ],

	[ "checkbutton_slovak",     "slovak",     "sk" ],
	[ "checkbutton_slovenian",  "slovenian",  "sl" ],
	[ "checkbutton_spanish",    "spanish",    "es" ],
	[ "checkbutton_swahili",    "swahili",    "sw" ],
	[ "checkbutton_swedish",    "swedish",    "sv" ],
	[ "checkbutton_thai",       "thai",       "th" ],
	[ "checkbutton_turkish",    "turkish",    "tr" ],
	[ "checkbutton_ukrainian",  "ukrainian",  "uk" ],
	[ "checkbutton_vietnamese", "vietnamese", "vi" ],
	[ "checkbutton_welsh",      "welsh",      "cy" ],

	[ "checkbutton_yiddish",    "yiddish",    "yi" ],
	]

class wind_main():

	def __init__( self, name_glade_file="wind_main.glade" ):
	# def __init__( self, name_glade_file="wind_main.glade", Form=None ):

		self.folder_list = []
		self.folder_list_scan = []
		self.folder_list_po = []
		self.list_parse = []
		self.bool_new_scan = False

		# Add language support
		_ = Language_Init.Translator(FConfig).lang.gettext
		self._ = _

		try:
			self.builder = Gtk.Builder()
			# Add language support from glade file
			self.builder.set_translation_domain("xbpf")
			self.builder.add_from_file( FConfig.DATADIR + "/" + name_glade_file )
		except:
			print("except from:Gtk.Builder() to:", name_glade_file )
			sys.exit(1)

		self.wind_main=self.builder.get_object("wind_main")

		dic={
			"on_wind_main_delete_event": self.on_wind_main_delete_event,
			"on_wind_main_destroy_event": self.on_wind_main_destroy_event,
			"on_imagemenuitem_quit_activate": self.on_imagemenuitem_quit_activate,
			"on_imagemenuitem_about_activate": self.on_imagemenuitem_about_activate,
			"on_notebook_main_switch_page": self.on_notebook_main_switch_page,
			"on_button_choice_folder_clicked": self.on_button_choice_folder_clicked,
			"on_button_lang_select_all_clicked": self.on_button_lang_select_all_clicked,
			"on_button_lang_deselect_all_clicked": self.on_button_lang_deselect_all_clicked,
			"on_button_lang_choice_by_default_clicked": self.on_button_lang_choice_by_default_clicked,
			"on_button_lang_get_from_folder_clicked": self.on_button_lang_get_from_folder_clicked,
			"on_button_generate_mo_files_clicked": self.on_button_generate_mo_files_clicked,
			"on_button_previous_clicked": self.on_button_previous_clicked,
			"on_button_next_clicked": self.on_button_next_clicked,
			"on_entry_file_process_changed": self.on_entry_file_process_changed,
			"on_button_choice_folder_realize": self.on_button_choice_folder_realize,
			"on_label_put_files_selected_realize": self.on_label_put_files_selected_realize,
		}
		self.builder.connect_signals(dic)

		self.label_presentation=self.builder.get_object("label_presentation")
		self.label_put_files_selected=self.builder.get_object("label_put_files_selected")
		self.entry_file_process=self.builder.get_object("entry_file_process")
		self.button_lang_select_all=self.builder.get_object("button_lang_select_all")
		self.button_lang_deselect_all=self.builder.get_object("button_lang_deselect_all")
		self.button_lang_choice_by_default=self.builder.get_object("button_lang_choice_by_default")
		self.button_lang_get_from_folder=self.builder.get_object("button_lang_get_from_folder")
		self.radiobutton_translation_create_only=self.builder.get_object("radiobutton_translation_create_only")
		self.radiobutton_translation_force=self.builder.get_object("radiobutton_translation_force")
		self.radiobutton_translation_complete=self.builder.get_object("radiobutton_translation_complete")
		self.button_generate_mo_files=self.builder.get_object("button_generate_mo_files")
		self.button_previous=self.builder.get_object("button_previous")
		self.button_next=self.builder.get_object("button_next")
		self.notebook_main=self.builder.get_object("notebook_main")
		self.button_choice_folder=self.builder.get_object("button_choice_folder")
		self.box_previous_next=self.builder.get_object("box_previous_next")
		self.box_progressbar=self.builder.get_object("box_progressbar")
		self.progressbar_main=self.builder.get_object("progressbar_main")
		self.label_progressbar=self.builder.get_object("label_progressbar")
		self.timeout_id =  -1
		self.box_radiobutton=self.builder.get_object("box_radiobutton")

		self.box_progressbar.hide()
		self.bool_error_make_msgfmt = False
		self.list_syntax_error = []

		# Preferences: init
		self.Prefs = Preferences()
		self.Prefs.Preferences_read_file()
		# Preferences: set window position
		win_width      = self.Prefs.Preferences_get_win_width()
		win_height     = self.Prefs.Preferences_get_win_height()
		main_win_pos_x = self.Prefs.Preferences_get_main_win_pos_x()
		main_win_pos_y = self.Prefs.Preferences_get_main_win_pos_y()
		self.wind_main.resize( win_width, win_height )
		self.wind_main.move( main_win_pos_x, main_win_pos_y )
		# Preferences:
		self.button_choice_folder.set_label( self.Prefs.Preferences_get_form_ProjectDestination() )
		self.read_folder( self.button_choice_folder.get_label() )
		self.print_files()

		self.entry_file_process.set_text( self.Prefs.Preferences_get_ExtentionChoice() )

		# init notebook in reverse
		list_page_notebook = [ 2, 1, 0 ]
		for page_notebook in list_page_notebook:
			self.notebook_main.set_current_page( page_notebook )
		# self.config_buttons_actions()

		# header
		title_string = "X Build Pot File"
		title_label  = _("Update, creation, translation and production of MO files.")
		string = "<span font_desc=\"sans bold 18 \"><b>\n%s\n</b></span><b>%s</b>\n\n" % (title_string, title_label)
		self.label_presentation.set_markup( string )

		self.wind_main.set_title("%s - Version %s" % (FConfig.__PACKAGE__, FConfig.__VERSION__))
		self.wind_main.set_icon_from_file( FConfig.DATADIR + "/xbpf.png" )
		self.wind_main.show()

	#
	# End of: Bye
	#
	def main_save_preferences(self):
		x, y = self.wind_main.get_position()
		width, height = self.wind_main.get_size()
		self.Prefs.Preferences_save_op(
			self.button_choice_folder.get_label(),
			self.entry_file_process.get_text(),
			None,
			x, y, width, height)

	def on_wind_main_delete_event(self, widget, data=None):
		self.main_save_preferences()
		Gtk.main_quit()

	def on_wind_main_destroy_event(self, widget, data=None):
		self.main_save_preferences()
		Gtk.main_quit()

	def on_imagemenuitem_quit_activate(self, widget, data=None):
		self.main_save_preferences()
		Gtk.main_quit()


	#
	# About
	#
	def on_imagemenuitem_about_activate(self, widget, data=None):
		wind_about(Form=self.wind_main)


	#
	# Updating buttons
	#
	def config_buttons_actions(self):

		new_page = self.notebook_main.get_current_page()
		if new_page == NOTEBOOK_CHOICE:
			self.button_next.set_sensitive( True )
			if not self.folder_list_scan:
				self.button_next.set_sensitive( False )
		elif new_page == NOTEBOOK_CHOICE_TRANSLATE:
			self.button_next.set_sensitive( False )
			for item in language_liste:
				if self.builder.get_object( item[ 0 ] ).get_active():
					self.button_next.set_sensitive( True )
					break

		if self.bool_new_scan == True:
			self.bool_new_scan = False

			# All langues deselect
			for item in language_liste:
				self.builder.get_object( item[ 0 ] ).set_active( False )

			for item in self.folder_list_po:
				BaseName = os.path.basename( item )
				Name = BaseName.split(".")[0]
				Ext  = BaseName.split(".")[-1]
				for lang in language_liste:
					if Name == lang[2]:
						self.builder.get_object( lang[ 0 ] ).set_active( True )
						break


	#
	# Buttons init with notebook_switch_page
	#
	def on_notebook_main_switch_page(self, obj1, obj2, page):
		if page == NOTEBOOK_CHOICE:
			self.button_previous.hide()
			self.button_next.show()
		elif page == NOTEBOOK_GENERATE:
			self.button_previous.show()
			self.button_next.hide()
		else:
			self.button_previous.show()
			self.button_next.show()
		self.config_buttons_actions()

	def on_button_previous_clicked(self, widget, data=None):
		self.notebook_main.prev_page()
		self.config_buttons_actions()

	def on_button_next_clicked(self, widget, data=None):
		self.notebook_main.next_page()
		self.config_buttons_actions()


	#
	# Read foldre
	#
	def get_liste_parse(self):
		liste_parse = []
		string = self.entry_file_process.get_text().replace( ' ', '' )
		string = string.replace( '.', '' )
		list_string = string.split(',')
		for item in list_string:
			if item != "":
				liste_parse.append( '.' + item )
		return liste_parse

	def print_files(self):
		self.folder_list_scan[:] = []
		self.folder_list_po[:] = []
		self.label_put_files_selected.set_markup( "" )
		txt = ""
		txt_dir = ""
		self.list_parse[:] = []
		self.list_parse = self.get_liste_parse()
		if self.list_parse:
			if self.folder_list:
				for item in self.folder_list:
					if os.path.isdir( item ) == True:
						txt_dir = "<b>" + item + "</b>" + "\n"
					else:
						fileName, fileExtension = os.path.splitext( item )
						if fileExtension in self.list_parse:
							BaseName = os.path.basename( item )
							Name = BaseName.split(".")[0]
							Ext  = BaseName.split(".")[-1]
							txt = txt + txt_dir + BaseName + "\n"
							txt_dir = ""
							# Get scan file list
							self.folder_list_scan.append( item )
						if fileExtension == '.po':
							self.folder_list_po.append( item )
				self.label_put_files_selected.set_markup( txt.strip() )
			else:
				self.label_put_files_selected.set_markup( "<b>" + _("Please select a correct folder.") + "</b>" )
		else:
			self.label_put_files_selected.set_markup( "<b>" + _("Please select a directory and a template.") + "</b>" )

	def read_folder( self, path ):
		self.bool_new_scan = True
		self.folder_list[:] = []
		cpt_folder = 0
		for root, dirs, files in os.walk( path ):
			if(files):
				self.folder_list.append( root )
				cpt_folder += 1
				for f in files:
					self.folder_list.append( root + '/' + f )
			if cpt_folder >= 25:
				self.folder_list[:] = []
				self.bool_new_scan = True
				break

	def on_button_choice_folder_clicked(self, widget, data=None):
		dialog = Gtk.FileChooserDialog(
			_("Please choose a folder"),
			None,
			Gtk.FileChooserAction.SELECT_FOLDER,
			(Gtk.STOCK_CANCEL,
			Gtk.ResponseType.CANCEL,
			_("Select"),
			Gtk.ResponseType.OK))
		dialog.set_default_size(400, 400)
		dialog.set_current_folder( self.button_choice_folder.get_label() )
		response = dialog.run()
		if response == Gtk.ResponseType.OK:
			self.button_choice_folder.set_label( dialog.get_filename() )
			self.read_folder( dialog.get_filename() )
		dialog.destroy()
		self.print_files()


	#
	# Choices parse files
	#
	def on_entry_file_process_changed(self, widget, data=None):
		self.print_files()
		self.config_buttons_actions()


	#
	# Choices from languages
	#
	def on_button_lang_select_all_clicked(self, widget, data=None):
		for item in language_liste:
			self.builder.get_object( item[ 0 ] ).set_active( True )
		self.config_buttons_actions()

	def on_button_lang_deselect_all_clicked(self, widget, data=None):
		for item in language_liste:
			self.builder.get_object( item[ 0 ] ).set_active( False )
		self.config_buttons_actions()

	def on_button_lang_choice_by_default_clicked(self, widget, data=None):
		self.builder.get_object( "checkbutton_english" ).set_active( True )
		self.builder.get_object( "checkbutton_french" ).set_active( True )
		self.builder.get_object( "checkbutton_german" ).set_active( True )
		self.builder.get_object( "checkbutton_italian" ).set_active( True )
		self.builder.get_object( "checkbutton_spanish" ).set_active( True )
		self.config_buttons_actions()

	def on_button_lang_get_from_folder_clicked(self, widget, data=None):
		for item in self.folder_list_po:
			BaseName = os.path.basename( item )
			Name = BaseName.split(".")[0]
			Ext  = BaseName.split(".")[-1]
			for lang in language_liste:
				if lang[2] == Name:
					self.builder.get_object( lang[ 0 ] ).set_active( True )
					break
		self.config_buttons_actions()


	#
	# Creation MO files
	#
	def create_project(self, PathDest):
		print("\nUpdate, creation, translation and production of MO files.")
		print("-------------------------------------------------------")

		ListPotFile = []
		for item in language_liste:
			if self.builder.get_object( item[ 0 ] ).get_active() == True:
				ListPotFile.append( item[2] + '.po' )

		BuildPotFile = build_pot_file(
					self.wind_main,
					PathDest,
					self.list_parse,
					ListPotFile,
					self.radiobutton_translation_complete.get_active(),
					self.radiobutton_translation_force.get_active()
					)
		self.list_syntax_error, self.bool_error_make_msgfmt = BuildPotFile.create_pot_file()
		self.timeout_id = -1

	def timeout_create_project(self, user_data):
		self.progressbar_main.pulse()
		if self.timeout_id > 0:
			return True
		else:
			self.button_previous.set_sensitive( True )
			self.box_radiobutton.set_sensitive( True )
			self.button_generate_mo_files.show()
			self.box_progressbar.hide()
			if self.bool_error_make_msgfmt == True or self.list_syntax_error:
				err_title_text = ""
				if self.list_syntax_error:
					err_title_text = "\nSyntax error in file:\n"
				for err in self.list_syntax_error:
					err_title_text += err + '\n'
				err_title_text += "\n" + _("Do not forget to correct errors in the text of PO files for MO files")
				wind_info(
					Form  = self.wind_main,
					Title =_("Syntax error in po file"),
					Text  = err_title_text )
			return False

	def thread_create_project(self, PathDest):
		self.button_previous.set_sensitive( False )
		self.box_radiobutton.set_sensitive( False )
		self.button_generate_mo_files.hide()
		self.box_progressbar.show()
		self.label_progressbar.set_label( _("Please wait while the treatment") )
		self.timeout_id = GObject.timeout_add( 100, self.timeout_create_project, None )
		thread = threading.Thread( target=self.create_project, args=(PathDest,))
		thread.start()

	def on_button_generate_mo_files_clicked(self, widget, data=None):
		print( self.radiobutton_translation_create_only.get_active() )
		print( self.radiobutton_translation_force.get_active() )
		print( self.radiobutton_translation_complete.get_active() )
		self.thread_create_project( self.button_choice_folder.get_label() )


	#
	# Drag and drop
	#
	def on_drag_data_received( self, widget, drag_context, x,y, data,info, time):
		for uri in data.get_uris():
			# Example uri: file:///home/cat/DEV/BASE_PYTHON/wind_main.glade
			name_file = uri[ 7: ]
			name_file = name_file.replace( '%20', ' ')
			if os.path.isdir( name_file ) == True:
				self.button_choice_folder.set_label( name_file )
				self.read_folder( name_file )
				self.print_files()
				self.config_buttons_actions()
				break

	def on_button_choice_folder_realize(self, widget, data=None):
		widget.drag_dest_set(
			Gtk.DestDefaults.MOTION |
			Gtk.DestDefaults.HIGHLIGHT |
			Gtk.DestDefaults.DROP,
			[Gtk.TargetEntry.new("text/uri-list", 0, 80)],
			Gdk.DragAction.COPY,
			)
		widget.connect("drag-data-received", self.on_drag_data_received)

	def on_label_put_files_selected_realize(self, widget, data=None):
		widget.drag_dest_set(
			Gtk.DestDefaults.MOTION |
			Gtk.DestDefaults.HIGHLIGHT |
			Gtk.DestDefaults.DROP,
			[Gtk.TargetEntry.new("text/uri-list", 0, 80)],
			Gdk.DragAction.COPY,
			)
		widget.connect("drag-data-received", self.on_drag_data_received)



if __name__ == '__main__':
	wind = wind_main()
	Gtk.main()
